//****************************************************************************************************************
// MotionTracker.ini is a simple Digital Oscilloscope for testing Motion Simulator tracking for 3 motors 
// written for UNO R3 by Craig Hoffmann
//          
//****************************************************************************************************************
// 
//
//                    +----+              +------+
//                  +-|PWR |--------------| USB  |-----+
//                  | |    |              |      |     |
//                  | +----+              +------+    o|
//                  |                                 o|
//                  |                                 o|AREF
//                  |                                 o|GND   
//                NC|o                                o|13    
//             IOREF|o   +----+                       o|12    
//             RESET|o   |    |                       o|11~   
//              3.3V|o   |    |                       o|10~   
//                5V|o   |    |    Arduino            o|9~    
//               GND|o   |    |      UNO              o|8
//  Test System  GND|o   |    |                        |
//               VIN|o   |    |                       o|7      
//                  |    |    |                       o|6~     
//  Motor 1 Pot   A0|o   |    |                       o|5~     
//  Motor 2 Pot   A1|o   |    |                       o|4      
//  Motor 3 Pot   A2|o   |    |                       o|3~     
//                A3|o   |    |                       o|2      
//                A4|o   |    |                       o|1
//                A5|o   +-/\-+                       o|0
//                  +-\                      /---------+
//                     \--------------------/
//
// 


// BOF preprocessor bug prevention - leave me at the top of the arduino-code
#if 1
__asm volatile ("nop");
#endif

#include <EEPROM.h>

// defines for setting and clearing register bits
#ifndef cbi
    #define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
    #define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#define PROCESS_PERIOD_uS 2000          // Send updates every 2ms
#define START_BYTE '['                  // Start Byte for serial commands
#define END_BYTE ']'                    // End Byte for serial commands


unsigned long TimesUp=0;           // Counter used to see if it's time to calculate next PID update

unsigned int Feedback1 = 511;
unsigned int Feedback2 = 511;
unsigned int Feedback3 = 511;

const int FeedbackPin1 = A0;   // Motor 1 feedback pin
const int FeedbackPin2 = A1;   // Motor 2 feedback pin
const int FeedbackPin3 = A2;   // Motor 3 feedback pin


//****************************************************************************************************************
//    Arduino setup subroutine called at startup/reset
//          
//****************************************************************************************************************

void setup()
{
    Serial.begin(500000);     

    // set analogue prescale to 16
    sbi(ADCSRA,ADPS2);
    cbi(ADCSRA,ADPS1);
    cbi(ADCSRA,ADPS0);
}





//****************************************************************************************************************
//  Send a data packet with the three feedback values to host computer
//          
//****************************************************************************************************************

void SendValues(int value1, int value2, int value3)
{
    int low,high;

    Serial.write(START_BYTE);
    high=value1/256;
    low=value1-(high*256);
    Serial.write(high);
    Serial.write(low);

    high=value2/256;
    low=value2-(high*256);
    Serial.write(high);
    Serial.write(low);

    high=value3/256;
    low=value3-(high*256);
    Serial.write(high);
    Serial.write(low);

    Serial.write(END_BYTE);
}



void TogglePin()
{
   static int PinOut=0;
   PinOut=1-PinOut;
   digitalWrite(8, PinOut); 
}


//****************************************************************************************************************
//    Main Arduino Program Loop
//          
//****************************************************************************************************************

void loop()
{

    // Initialise the PID ready timer

    TimesUp = micros();
    
    // Main Program loop

    while (1==1) 
    {

        // Wait until its time to send the next position updates

        while ((micros() - TimesUp) < PROCESS_PERIOD_uS) { ; }
        TimesUp += PROCESS_PERIOD_uS;
        TogglePin();                      // Used for testing 

        // Read the POTs

        Feedback1 = analogRead(FeedbackPin1);
        Feedback2 = analogRead(FeedbackPin2);
        Feedback3 = analogRead(FeedbackPin3);

        // Send the data

        SendValues(Feedback1,Feedback2,Feedback3);
    }
}
